@@ -20,10 +20,24 @@ new_proj_and_gdal <- function() {
     PROJis6ormore() && GDALis3ormore()
 }
 
-is_proj_network_enabled <- function() {
+is_proj_CDN_enabled <- function() {
     .Call("proj_network_enabled", PACKAGE="rgdal")
 }
 
+enable_proj_CDN <- function() {
+    .Call("enable_proj_network", PACKAGE="rgdal")
+    paste0("Using: ", proj_CDN_user_writable_dir(), sep="")
+}
+
+disable_proj_CDN <- function() {
+    invisible(.Call("disable_proj_network", PACKAGE="rgdal"))
+}
+
+proj_CDN_user_writable_dir <- function() {
+    .Call("get_proj_user_writable_dir", PACKAGE="rgdal")
+}
+
+
 GDAL_OSR_PROJ <- function() {
     res <- .Call("R_GDAL_OSR_PROJ", PACKAGE="rgdal")
     if (is.null(res)) return(res)
@@ -36,23 +50,33 @@ GDAL_OSR_PROJ <- function() {
 
 getPROJ4libPath <- function() {
     res <- Sys.getenv("PROJ_LIB")
-    if (new_proj_and_gdal()) {
+    if (PROJis6ormore()) {
         attr(res, "search_path") <- .Call("get_proj_search_path",
             PACKAGE="rgdal")
     }
     res
 }
 
-set_proj_search_paths <- function(paths) {
-    PV <- .Call("PROJ4VersionInfo", PACKAGE="rgdal")[[2]]
-    if (PV >= 600 && PV < 700) {
-        stopifnot(is.character(paths))
-        stopifnot(length(paths) == 1)
-        stopifnot(dir.exists(paths[1]))
-# length restricted to 1 for PROJ 6
-        res <- .Call("set_proj_search_path", paths[1], PACKAGE="rgdal")
+get_proj_search_paths <- function() {
+    if (PROJis6ormore()) {
+        res <- .Call("get_proj_search_path", PACKAGE="rgdal")
+        res <- strsplit(res, .Platform$path.sep)[[1]]
     } else {
-        res <- ""
+        res <- NULL
+    }
+    res
+}
+
+set_proj_search_paths <- function(paths) {
+    if (PROJis6ormore()) {
+        stopifnot(!missing(paths))
+        stopifnot(is.character(paths))
+        stopifnot(length(paths) > 0)
+        n <- length(paths)
+        for (i in 1:n) stopifnot(dir.exists(paths[i]))
+        res <- .Call("set_proj_paths", paths, PACKAGE="rgdal")
+    } else {
+        res <- NULL
     }
     res
 }
@@ -87,7 +111,37 @@ get_last_coordOp <- function() {
     get(".last_coordOp", envir=.RGDAL_CACHE)
 }
 
-"project" <- function(xy, proj, inv=FALSE, use_ob_tran=FALSE, legacy=TRUE, allowNAs_if_not_legacy=FALSE, coordOp=NULL, verbose=FALSE) {
+get_aoi <- function(obj, xy, inv, proj) {
+    if (missing(obj)) { # used in project
+        bb <- cbind(range(xy[,1], na.rm=TRUE), range(xy[,2], na.rm=TRUE))
+        if (inv) {
+            o <- project(bb, proj, inv=inv, use_aoi=FALSE)
+        } else {
+            o <- bb
+        }
+    } else { # used in spTransform
+        if (is.projected(obj)) { 
+            tg <- wkt(obj)
+            if (is.null(tg)) {
+                tg <- proj4string(obj)
+                if (length(grep("+init", tg)) > 0L) {
+                    tg <- wkt(CRS(tg))
+                }
+            }
+            o <- project(t(bbox(obj))[,1:2], tg, inv=TRUE, use_aoi=FALSE)
+        } else {
+            o <- t(bbox(obj))[,1:2]
+        }
+    }
+    if (any(!is.finite(c(o)))) return(NULL)
+    aoi <- o + c(-0.1, +0.1) # stretch envelope
+    aoi[,1] <- sapply(aoi[,1], function(x) { if (x > 180) 180 - x else if (x < -180) 360 + x else x})
+    aoi[,2] <- sapply(aoi[,2], function(y) { if (y > 90) 90 else if (y < -90) -90 else y}) # constrain to -180/180, -90/90
+    c(t(aoi))
+}
+
+
+"project" <- function(xy, proj, inv=FALSE, use_ob_tran=FALSE, legacy=TRUE, allowNAs_if_not_legacy=FALSE, coordOp=NULL, verbose=FALSE, use_aoi=TRUE) {
 
 #    if (new_proj_and_gdal()) 
 #        warning("project() will not be adapted for PROJ 6 and is deprecated")
@@ -170,9 +224,16 @@ get_last_coordOp <- function() {
                     if (length(grep(" \\+type=crs", proj)) == 0L)
                         proj <- paste0(proj, " +type=crs")
                 }
-                coordOp <- .Call("project_ng_coordOp", proj, as.logical(inv),
-                    #as.logical(use_ob_tran), 
-                    PACKAGE="rgdal")
+                aoi <- NULL
+                if (!use_ob_tran && use_aoi) {
+                    aoi <- get_aoi(xy=xy, inv=inv, proj=proj)
+                    if (!is.null(aoi)) {
+                        stopifnot(length(aoi) == 4)
+                        aoi <- as.numeric(aoi)
+                    }
+                }
+                coordOp <- .Call("project_ng_coordOp", proj,
+                    as.logical(inv), aoi, PACKAGE="rgdal")
             }
             if (verbose) cat(strwrap(coordOp), sep="\n")
             res <- .Call("project_ng",
@@ -321,6 +382,24 @@ if (!isGeneric("spTransform"))
         if (!is.null(dots$coordOp)) {
             coordOp <- dots$coordOp
         }
+        
+        use_aoi <- TRUE
+        if (!is.null(dots$use_aoi)) {
+            use_aoi <- dots$use_aoi
+            if (use_ob_tran != 0L) use_aoi <- FALSE
+            if (!is.null(coordOp)) use_aoi <- FALSE
+            if (!new_proj_and_gdal()) use_aoi <- FALSE
+        }
+        
+        aoi <- NULL
+        if (use_aoi) {
+            aoi <- get_aoi(x)
+            if (!is.null(aoi)) {
+                stopifnot(length(aoi) == 4)
+                aoi <- as.numeric(aoi)
+            }
+        }
+
 	crds <- coordinates(x)
 	crds.names <- dimnames(crds)[[2]] # crds is matrix
 	n <- nrow(crds)
@@ -330,7 +409,7 @@ if (!isGeneric("spTransform"))
                 if (use_ob_tran == 0L) {
                     attr(n, "enforce_xy") <- enforce_xy
                     res <- .Call("transform_ng", from_args, to_args, coordOp, n,
-		        as.double(crds[,1]), as.double(crds[,2]), NULL,
+		        as.double(crds[,1]), as.double(crds[,2]), NULL, aoi,
 		        PACKAGE="rgdal")
                     out_coordOp <- res[[5]]
                 } else {
@@ -347,7 +426,7 @@ if (!isGeneric("spTransform"))
                     else use_ob_tran1 <- TRUE
                     if (is.null(coordOp)) {
                         out_coordOp <- .Call("project_ng_coordOp", proj,
-                            as.logical(inv), #as.logical(use_ob_tran1),
+                            as.logical(inv), NULL, #as.logical(use_ob_tran1),
                             PACKAGE="rgdal")
                     }
                     res <- .Call("project_ng", as.integer(n), 
@@ -373,7 +452,7 @@ if (!isGeneric("spTransform"))
                     attr(n, "enforce_xy") <- enforce_xy
                     res <- .Call("transform_ng", from_args, to_args, coordOp, n,
 		        as.double(crds[,1]), as.double(crds[,2]),
-                        as.double(crds[,3]), PACKAGE="rgdal")
+                        as.double(crds[,3]), aoi, PACKAGE="rgdal")
                     out_coordOp <- res[[6]]
                 } else {
                     if (use_ob_tran == -1L) {
@@ -389,7 +468,7 @@ if (!isGeneric("spTransform"))
                     else use_ob_tran1 <- TRUE
                     if (is.null(coordOp)) {
                         out_coordOp <- .Call("project_ng_coordOp", proj,
-                            as.logical(inv), #as.logical(use_ob_tran1),
+                            as.logical(inv), NULL, #as.logical(use_ob_tran1),
                             PACKAGE="rgdal")
                     }
                     res <- .Call("project_ng", as.integer(n), 
@@ -456,7 +535,7 @@ setMethod("spTransform", signature("SpatialGridDataFrame", "CRS"),
 
 
 ".spTransform_Line" <- function(x, to_args, from_args, ii, jj,
-                use_ob_tran, coordOp, enforce_xy) {
+                use_ob_tran, coordOp, enforce_xy, aoi) {
 	crds <- slot(x, "coords")
 	n <- nrow(crds)
         attr(n, "ob_tran") <- use_ob_tran
@@ -464,7 +543,7 @@ setMethod("spTransform", signature("SpatialGridDataFrame", "CRS"),
             if (use_ob_tran == 0L) {
                 attr(n, "enforce_xy") <- enforce_xy
                 res <- .Call("transform_ng", from_args, to_args, coordOp, n,
-		    as.double(crds[,1]), as.double(crds[,2]), NULL,
+		    as.double(crds[,1]), as.double(crds[,2]), NULL, aoi,
 		    PACKAGE="rgdal")
                 out_coordOp <- res[[5]]
             } else {
@@ -482,7 +561,7 @@ setMethod("spTransform", signature("SpatialGridDataFrame", "CRS"),
                 else use_ob_tran1 <- TRUE
                 if (is.null(coordOp)) {
                     coordOp <- .Call("project_ng_coordOp", proj,
-                        as.logical(inv), #as.logical(use_ob_tran1),
+                        as.logical(inv), NULL, #as.logical(use_ob_tran1),
                         PACKAGE="rgdal")
                 }
                 res <- .Call("project_ng", as.integer(n), 
@@ -512,7 +591,7 @@ setMethod("spTransform", signature("SpatialGridDataFrame", "CRS"),
 #setMethod("spTransform", signature("Sline", "CRS"), spTransform.Sline)
 
 ".spTransform_Lines" <- function(x, to_args, from_args, ii,
-                use_ob_tran, coordOp, enforce_xy) {
+                use_ob_tran, coordOp, enforce_xy, aoi) {
 	ID <- slot(x, "ID")
 	input <- slot(x, "Lines")
 	n <- length(input)
@@ -521,7 +600,8 @@ setMethod("spTransform", signature("SpatialGridDataFrame", "CRS"),
 	for (i in 1:n) {
             output[[i]] <- .spTransform_Line(input[[i]], 
 		to_args=to_args, from_args=from_args, ii=ii, jj=i,
-                use_ob_tran=use_ob_tran, coordOp=coordOp, enforce_xy=enforce_xy)
+                use_ob_tran=use_ob_tran, coordOp=coordOp,
+                enforce_xy=enforce_xy, aoi=aoi)
             if (is.null(coordOp) && new_proj_and_gdal() && i == 1) {
                 out_coordOp <- attr(output[[i]], "coordOp")
                 coordOp <-  gsub(" ", " +", paste0("+", out_coordOp))
@@ -618,13 +698,31 @@ setMethod("spTransform", signature("SpatialGridDataFrame", "CRS"),
         if (!is.null(dots$coordOp)) {
             coordOp <- dots$coordOp
         }
+        use_aoi <- TRUE
+        if (!is.null(dots$use_aoi)) {
+            use_aoi <- dots$use_aoi
+            if (use_ob_tran != 0L) use_aoi <- FALSE
+            if (!is.null(coordOp)) use_aoi <- FALSE
+            if (!new_proj_and_gdal()) use_aoi <- FALSE
+        }
+        
+        aoi <- NULL
+        if (use_aoi) {
+            aoi <- get_aoi(x)
+            if (!is.null(aoi)) {
+                stopifnot(length(aoi) == 4)
+                aoi <- as.numeric(aoi)
+            }
+        }
+
 	input <- slot(x, "lines")
 	n <- length(input)
 	output <- vector(mode="list", length=n)
 	for (i in 1:n) {
             output[[i]] <- .spTransform_Lines(input[[i]], 
 		to_args=to_args, from_args=from_args, ii=i,
-                use_ob_tran=use_ob_tran, coordOp=coordOp, enforce_xy=enforce_xy)
+                use_ob_tran=use_ob_tran, coordOp=coordOp,
+                enforce_xy=enforce_xy, aoi=aoi)
             if (is.null(coordOp) && new_proj_and_gdal() && i == 1) {
                 out_coordOp <- attr(output[[i]], "coordOp")
                 coordOp <-  gsub(" ", " +", paste0("+", out_coordOp))
@@ -650,7 +748,7 @@ setMethod("spTransform", signature("SpatialLinesDataFrame", "CRS"), spTransform.
 
 
 ".spTransform_Polygon" <- function(x, to_args, from_args, ii, jj,
-                use_ob_tran, coordOp, enforce_xy) {
+                use_ob_tran, coordOp, enforce_xy, aoi) {
 	crds <- slot(x, "coords")
 	n <- nrow(crds)
         attr(n, "ob_tran") <- use_ob_tran
@@ -658,7 +756,7 @@ setMethod("spTransform", signature("SpatialLinesDataFrame", "CRS"), spTransform.
             if (use_ob_tran == 0L) {
                 attr(n, "enforce_xy") <- enforce_xy
                 res <- .Call("transform_ng", from_args, to_args, coordOp, n,
-		    as.double(crds[,1]), as.double(crds[,2]), NULL,
+		    as.double(crds[,1]), as.double(crds[,2]), NULL, aoi,
 		    PACKAGE="rgdal")
                 out_coordOp <- res[[5]]
             } else {
@@ -676,7 +774,7 @@ setMethod("spTransform", signature("SpatialLinesDataFrame", "CRS"), spTransform.
                 else use_ob_tran1 <- TRUE
                 if (is.null(coordOp)) {
                     coordOp <- .Call("project_ng_coordOp", proj,
-                        as.logical(inv), #as.logical(use_ob_tran1),
+                        as.logical(inv), NULL, #as.logical(use_ob_tran1),
                         PACKAGE="rgdal")
                 }
                 res <- .Call("project_ng", as.integer(n), 
@@ -705,7 +803,7 @@ setMethod("spTransform", signature("SpatialLinesDataFrame", "CRS"), spTransform.
 
 
 ".spTransform_Polygons" <- function(x, to_args, from_args, ii,
-                use_ob_tran, coordOp, enforce_xy) {
+                use_ob_tran, coordOp, enforce_xy, aoi) {
 	ID <- slot(x, "ID")
 	input <- slot(x, "Polygons")
 	n <- length(input)
@@ -714,7 +812,8 @@ setMethod("spTransform", signature("SpatialLinesDataFrame", "CRS"), spTransform.
 	for (i in 1:n) {
             output[[i]] <- .spTransform_Polygon(input[[i]], 
 		to_args=to_args, from_args=from_args, ii=ii, jj=i,
-                use_ob_tran=use_ob_tran, coordOp=coordOp, enforce_xy=enforce_xy)
+                use_ob_tran=use_ob_tran, coordOp=coordOp,
+                enforce_xy=enforce_xy, aoi=aoi)
             if (is.null(coordOp) && new_proj_and_gdal() && i == 1) {
                 out_coordOp <- attr(output[[i]], "coordOp")
                 coordOp <-  gsub(" ", " +", paste0("+", out_coordOp))
@@ -811,13 +910,31 @@ setMethod("spTransform", signature("SpatialLinesDataFrame", "CRS"), spTransform.
         if (!is.null(dots$coordOp)) {
             coordOp <- dots$coordOp
         }
+        use_aoi <- TRUE
+        if (!is.null(dots$use_aoi)) {
+            use_aoi <- dots$use_aoi
+            if (use_ob_tran != 0L) use_aoi <- FALSE
+            if (!is.null(coordOp)) use_aoi <- FALSE
+            if (!new_proj_and_gdal()) use_aoi <- FALSE
+        }
+        
+        aoi <- NULL
+        if (use_aoi) {
+            aoi <- get_aoi(x)
+            if (!is.null(aoi)) {
+                stopifnot(length(aoi) == 4)
+                aoi <- as.numeric(aoi)
+            }
+        }
+
 	input <- slot(x, "polygons")
 	n <- length(input)
 	output <- vector(mode="list", length=n)
 	for (i in 1:n) {
             output[[i]] <- .spTransform_Polygons(input[[i]], 
 		to_args=to_args, from_args=from_args, ii=i,
-                use_ob_tran=use_ob_tran, coordOp=coordOp, enforce_xy=enforce_xy)
+                use_ob_tran=use_ob_tran, coordOp=coordOp,
+                enforce_xy=enforce_xy, aoi=aoi)
             if (is.null(coordOp) && new_proj_and_gdal() && i == 1) {
                 out_coordOp <- attr(output[[i]], "coordOp")
                 coordOp <-  gsub(" ", " +", paste0("+", out_coordOp))
